@@ -2,7 +2,7 @@ from typing import Union, Callable
 
 from casadi import MX, SX, integrator as casadi_integrator, horzcat, Function
 
-from .integrator import RK4, RK8, IRK, COLLOCATION
+from .integrator import RK4, RK8, IRK, COLLOCATION, CVODES
 from ..misc.enums import ControlType
 
 
@@ -14,9 +14,14 @@ class OdeSolverBase:
     ----------
     steps: int
         The number of integration steps
+    steps_scipy: int
+        Number of steps while integrating with scipy
     rk_integrator: Union[RK4, RK8, IRK]
         The corresponding integrator class
-
+    is_direct_collocation: bool
+        indicating if the ode solver is direct collocation method
+    is_direct_shooting: bool
+        indicating if the ode solver is direct shooting method
     Methods
     -------
     integrator(self, ocp, nlp) -> list
@@ -27,7 +32,7 @@ class OdeSolverBase:
 
     def __init__(self):
         self.steps = 1
-        self.steps_scipy = 5  # Number of steps while integrating with scipy
+        self.steps_scipy = 5
         self.rk_integrator = None
         self.is_direct_collocation = False
         self.is_direct_shooting = False
@@ -292,7 +297,7 @@ class OdeSolver:
             """
 
             if ocp.cx is SX:
-                raise NotImplementedError("use_sx=True and OdeSolver.IRK are not yet compatible")
+                raise RuntimeError("use_sx=True and OdeSolver.IRK are not yet compatible")
 
             return super(OdeSolver.IRK, self).integrator(ocp, nlp)
 
@@ -303,6 +308,10 @@ class OdeSolver:
 
         def __init__(self):
             super(OdeSolver.CVODES, self).__init__()
+            self.rk_integrator = CVODES
+            self.is_direct_collocation = False
+            self.is_direct_shooting = True
+            self.steps = 1
 
         def integrator(self, ocp, nlp) -> list:
             """
@@ -320,7 +329,7 @@ class OdeSolver:
             A list of integrators
             """
             if not isinstance(ocp.cx(), MX):
-                raise RuntimeError("CVODES integrator can only be used with MX graphs")
+                raise RuntimeError("use_sx=True and OdeSolver.CVODES are not yet compatible")
             if ocp.v.parameters_in_list.shape != 0:
                 raise RuntimeError("CVODES cannot be used while optimizing parameters")
             if nlp.external_forces:
