@@ -28,7 +28,7 @@ from ..limits.path_conditions import InterpolationType
 from ..limits.penalty import PenaltyOption
 from ..limits.objective_functions import ObjectiveFunction
 from ..misc.__version__ import __version__
-from ..misc.enums import ControlType, SolverType, Shooting, PlotType, CostType
+from ..misc.enums import ControlType, SolverType, Shooting, PlotType, CostType, SolutionIntegrator
 from ..misc.mapping import BiMappingList, Mapping
 from ..misc.utils import check_version
 from ..optimization.parameters import ParameterList, Parameter
@@ -749,7 +749,7 @@ class OptimalControlProgram:
         automatically_organize: bool = True,
         show_bounds: bool = False,
         shooting_type: Shooting = Shooting.MULTIPLE,
-        use_scipy_integrator: bool = False,
+        integrator: SolutionIntegrator = SolutionIntegrator.DEFAULT,
     ) -> PlotOcp:
         """
         Create all the plots associated with the OCP
@@ -762,8 +762,8 @@ class OptimalControlProgram:
             If the ylim should fit the bounds
         shooting_type: Shooting
             What type of integration
-        use_scipy_integrator: bool
-            Use the scipy solve_ivp integrator for RungeKutta 45 instead of currently defined integrator
+        integrator: SolutionIntegrator
+            Use the ode defined by OCP or use a separate integrator provided by scipy
 
         Returns
         -------
@@ -775,7 +775,7 @@ class OptimalControlProgram:
             automatically_organize=automatically_organize,
             show_bounds=show_bounds,
             shooting_type=shooting_type,
-            use_scipy_integrator=use_scipy_integrator,
+            integrator=integrator,
         )
 
     def solve(
